/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

describe('DeploymentAddCredentials', function () {
  var acl, clouds, users, jem, refs;

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('deployment-add-credentials', function () {
      done();
    });
  });

  beforeEach(function () {
    acl = { isReadOnly: sinon.stub().returns(false) };
  });

  beforeEach(function () {
    users = {
      jem: {
        user: 'foo'
      }
    };
    jem = {
      addTemplate: sinon.stub(),
      listRegions: function listRegions(cloud, cb) {
        return cb(null, ['us-east-1']);
      }
    };
    clouds = {
      aws: {
        id: 'aws',
        signupUrl: 'https://portal.aws.amazon.com/gp/aws/developer/' + 'registration/index.html',
        svgHeight: 48,
        svgWidth: 120,
        title: 'Amazon Web Services'
      },
      google: {
        id: 'google',
        signupUrl: 'https://console.cloud.google.com/billing/freetrial',
        svgHeight: 33,
        svgWidth: 256,
        title: 'Google Compute Engine'
      },
      azure: {
        id: 'azure',
        signupUrl: 'https://azure.microsoft.com/en-us/free/',
        svgHeight: 24,
        svgWidth: 204,
        title: 'Microsoft Azure'
      }
    };
    refs = {
      templateAccessKey: {
        validate: sinon.stub().returns(true),
        getValue: sinon.stub().returns('templateAccessKey')
      },
      templateName: {
        validate: sinon.stub().returns(true),
        getValue: sinon.stub().returns('templateName')
      },
      templateSecretKey: {
        validate: sinon.stub().returns(true),
        getValue: sinon.stub().returns('templateSecretKey')
      },
      selectRegion: {
        selectedIndex: 0,
        options: [{ value: 'us-east-1' }]
      }
    };
  });

  it('can render for aws', function () {
    var cloud = clouds['aws'];
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentAddCredentials, {
      acl: acl,
      changeState: sinon.stub(),
      controller: 'my-controller',
      cloud: cloud,
      jem: jem,
      setDeploymentInfo: sinon.stub(),
      users: users,
      validateForm: sinon.stub().returns(true) }), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    var buttons = [{
      action: instance._handleChangeCloud,
      title: 'Change cloud',
      type: 'inline-neutral'
    }, {
      title: 'Add credential',
      action: instance._handleAddCredentials,
      disabled: false,
      type: 'inline-positive'
    }];
    var expected = React.createElement(
      'div',
      { className: 'deployment-panel__child' },
      React.createElement(
        juju.components.DeploymentPanelContent,
        {
          title: 'Configure Amazon Web Services' },
        React.createElement(
          'div',
          { className: 'deployment-add-credentials__logo' },
          React.createElement(juju.components.SvgIcon, {
            height: cloud.svgHeight,
            name: cloud.id,
            width: cloud.svgWidth })
        ),
        React.createElement(
          'div',
          { className: 'twelve-col deployment-add-credentials__signup' },
          React.createElement(
            'a',
            { href: cloud.signupUrl,
              target: '_blank' },
            'Sign up for ',
            cloud.title,
            ' ',
            React.createElement(juju.components.SvgIcon, {
              name: 'external-link-16',
              size: '12' })
          )
        ),
        React.createElement(
          'form',
          { className: 'twelve-col' },
          React.createElement(
            'div',
            { className: 'six-col' },
            React.createElement(juju.components.DeploymentInput, {
              disabled: false,
              label: 'Credential name',
              placeholder: 'AWS-1',
              required: true,
              ref: 'templateName',
              validate: [{
                regex: /\S+/,
                error: 'This field is required.'
              }, {
                regex: /^[a-zA-Z0-9]([a-zA-Z0-9.-]*[a-zA-Z0-9])?$/,
                error: 'This field must only contain upper and lowercase ' + 'letters, numbers, and hyphens. It must not start or ' + 'end with a hyphen.'
              }] }),
            React.createElement(
              'select',
              { ref: 'selectRegion',
                disabled: false },
              React.createElement(
                'option',
                null,
                'Choose a region'
              ),
              [React.createElement(
                'option',
                { key: 'us-east-1', value: 'us-east-1' },
                'us-east-1'
              )]
            )
          ),
          React.createElement(
            'div',
            { className: 'deployment-panel__notice six-col last-col' },
            React.createElement(
              'p',
              { className: 'deployment-panel__notice-content' },
              React.createElement(juju.components.SvgIcon, {
                name: 'general-action-blue',
                size: '16' }),
              'Credentials are stored securely on our servers and we will notify you by email whenever they are used. See where they are used and manage or remove them via the account page.'
            )
          ),
          React.createElement(
            'h3',
            { className: 'deployment-panel__section-title twelve-col' },
            'Enter credentials'
          ),
          React.createElement(
            'div',
            { className: 'six-col' },
            React.createElement(
              'p',
              { className: 'deployment-add-credentials__p' },
              'You can obtain your AWS credentials at:',
              React.createElement('br', null),
              React.createElement(
                'a',
                { className: 'deployment-panel__link',
                  href: 'https://console.aws.amazon.com/iam/home?region=' + 'eu-west-1#security_credential',
                  target: '_blank' },
                'https://console.aws.amazon.com/iam/home?region=eu-west-1# security_credential'
              )
            ),
            React.createElement(juju.components.DeploymentInput, {
              disabled: false,
              label: 'Access key',
              placeholder: 'TDFIWNDKF7UW6DVGX98X',
              required: true,
              ref: 'templateAccessKey',
              validate: [{
                regex: /\S+/,
                error: 'This field is required.'
              }] }),
            React.createElement(juju.components.DeploymentInput, {
              disabled: false,
              label: 'Secret key',
              placeholder: 'p/hdU8TnOP5D7JNHrFiM8IO8f5GN6GhHj7tueBN9',
              required: true,
              ref: 'templateSecretKey',
              validate: [{
                regex: /\S+/,
                error: 'This field is required.'
              }] })
          )
        )
      ),
      React.createElement(juju.components.DeploymentPanelFooter, {
        buttons: buttons })
    );
    assert.deepEqual(output, expected);
  });

  it('can render for google', function () {
    var cloud = clouds['google'];
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentAddCredentials, {
      acl: acl,
      changeState: sinon.stub(),
      controller: 'my-controller',
      cloud: cloud,
      jem: jem,
      setDeploymentInfo: sinon.stub(),
      users: users,
      validateForm: sinon.stub().returns(true) }), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    var buttons = [{
      action: instance._handleChangeCloud,
      title: 'Change cloud',
      type: 'inline-neutral'
    }, {
      title: 'Add credential',
      action: instance._handleAddCredentials,
      disabled: true,
      type: 'inline-positive'
    }];
    var expected = React.createElement(
      'div',
      { className: 'deployment-panel__child' },
      React.createElement(
        juju.components.DeploymentPanelContent,
        {
          title: 'Configure Google Compute Engine' },
        React.createElement(
          'div',
          { className: 'deployment-add-credentials__logo' },
          React.createElement(juju.components.SvgIcon, {
            height: cloud.svgHeight,
            name: cloud.id,
            width: cloud.svgWidth })
        ),
        React.createElement(
          'div',
          { className: 'twelve-col deployment-add-credentials__signup' },
          React.createElement(
            'a',
            { href: cloud.signupUrl,
              target: '_blank' },
            'Sign up for ',
            cloud.title,
            ' ',
            React.createElement(juju.components.SvgIcon, {
              name: 'external-link-16',
              size: '12' })
          )
        ),
        React.createElement(
          'form',
          { className: 'twelve-col' },
          React.createElement(
            'div',
            { className: 'six-col' },
            React.createElement(juju.components.DeploymentInput, {
              disabled: false,
              label: 'Project ID (credential name)',
              placeholder: 'AWS-1',
              required: true,
              ref: 'templateName',
              validate: [{
                regex: /\S+/,
                error: 'This field is required.'
              }, {
                regex: /^[a-zA-Z0-9]([a-zA-Z0-9.-]*[a-zA-Z0-9])?$/,
                error: 'This field must only contain upper and lowercase ' + 'letters, numbers, and hyphens. It must not start or ' + 'end with a hyphen.'
              }] }),
            React.createElement(
              'select',
              { ref: 'selectRegion',
                disabled: false },
              React.createElement(
                'option',
                null,
                'Choose a region'
              ),
              [React.createElement(
                'option',
                { key: 'us-east-1', value: 'us-east-1' },
                'us-east-1'
              )]
            )
          ),
          React.createElement(
            'div',
            { className: 'deployment-panel__notice six-col last-col' },
            React.createElement(
              'p',
              { className: 'deployment-panel__notice-content' },
              React.createElement(juju.components.SvgIcon, {
                name: 'general-action-blue',
                size: '16' }),
              'Credentials are stored securely on our servers and we will notify you by email whenever they are used. See where they are used and manage or remove them via the account page.'
            )
          ),
          React.createElement(
            'h3',
            { className: 'deployment-panel__section-title twelve-col' },
            'Enter credentials'
          ),
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement(
              'p',
              { className: 'deployment-add-credentials__p six-col last-col' },
              'The GCE provider uses OAauth to Authenticate. This requires that you set it up and get the relevant credentials. For more information see ',
              React.createElement(
                'a',
                { className: 'deployment-panel__link',
                  href: 'https://cloud.google.com/copmute/dosc/api/how-tos/' + 'authorization',
                  target: '_blank' },
                'https://cloud.google.com/copmute/dosc/api/how-tos/ authorization'
              ),
              '. The key information can be downloaded as a JSON file, or copied from ',
              React.createElement(
                'a',
                { className: 'deployment-panel__link',
                  href: 'https://console.developers.google.com/project/apiui/' + 'credential',
                  target: '_blank' },
                'https://console.developers.google.com/project/apiui/credential'
              ),
              '.'
            ),
            React.createElement(
              'div',
              { className: 'deployment-add-credentials__upload twelve-col' },
              'Upload GCE auth-file or ',
              React.createElement(
                'span',
                { className: 'link' },
                'manually set the individual fields'
              )
            )
          )
        )
      ),
      React.createElement(juju.components.DeploymentPanelFooter, {
        buttons: buttons })
    );
    assert.deepEqual(output, expected);
  });

  it('can render for azure', function () {
    var cloud = clouds['azure'];
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentAddCredentials, {
      acl: acl,
      changeState: sinon.stub(),
      controller: 'my-controller',
      cloud: cloud,
      jem: jem,
      setDeploymentInfo: sinon.stub(),
      users: users,
      validateForm: sinon.stub().returns(true) }), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    var buttons = [{
      action: instance._handleChangeCloud,
      title: 'Change cloud',
      type: 'inline-neutral'
    }, {
      title: 'Add credential',
      action: instance._handleAddCredentials,
      disabled: true,
      type: 'inline-positive'
    }];
    var expected = React.createElement(
      'div',
      { className: 'deployment-panel__child' },
      React.createElement(
        juju.components.DeploymentPanelContent,
        {
          title: 'Configure Microsoft Azure' },
        React.createElement(
          'div',
          { className: 'deployment-add-credentials__logo' },
          React.createElement(juju.components.SvgIcon, {
            height: cloud.svgHeight,
            name: cloud.id,
            width: cloud.svgWidth })
        ),
        React.createElement(
          'div',
          { className: 'twelve-col deployment-add-credentials__signup' },
          React.createElement(
            'a',
            { href: cloud.signupUrl,
              target: '_blank' },
            'Sign up for ',
            cloud.title,
            ' ',
            React.createElement(juju.components.SvgIcon, {
              name: 'external-link-16',
              size: '12' })
          )
        ),
        React.createElement(
          'form',
          { className: 'twelve-col' },
          React.createElement(
            'div',
            { className: 'six-col' },
            React.createElement(juju.components.DeploymentInput, {
              disabled: false,
              label: 'Credential name',
              placeholder: 'AWS-1',
              required: true,
              ref: 'templateName',
              validate: [{
                regex: /\S+/,
                error: 'This field is required.'
              }, {
                regex: /^[a-zA-Z0-9]([a-zA-Z0-9.-]*[a-zA-Z0-9])?$/,
                error: 'This field must only contain upper and lowercase ' + 'letters, numbers, and hyphens. It must not start or ' + 'end with a hyphen.'
              }] }),
            React.createElement(
              'select',
              { ref: 'selectRegion',
                disabled: false },
              React.createElement(
                'option',
                null,
                'Choose a region'
              ),
              [React.createElement(
                'option',
                { key: 'us-east-1', value: 'us-east-1' },
                'us-east-1'
              )]
            )
          ),
          React.createElement(
            'div',
            { className: 'deployment-panel__notice six-col last-col' },
            React.createElement(
              'p',
              { className: 'deployment-panel__notice-content' },
              React.createElement(juju.components.SvgIcon, {
                name: 'general-action-blue',
                size: '16' }),
              'Credentials are stored securely on our servers and we will notify you by email whenever they are used. See where they are used and manage or remove them via the account page.'
            )
          ),
          React.createElement(
            'h3',
            { className: 'deployment-panel__section-title twelve-col' },
            'Enter credentials'
          ),
          React.createElement(
            'div',
            { className: 'twelve-col' },
            React.createElement(
              'p',
              { className: 'deployment-add-credentials__p six-col last-col' },
              'The following fields require your Windows Azure management information. For more information please see: ',
              React.createElement(
                'a',
                { className: 'deployment-panel__link',
                  href: 'https://msdn.microsoft.com/en-us/library/windowsazure',
                  target: '_blank' },
                'https://msdn.microsoft.com/en-us/library/windowsazure'
              ),
              ' for details.'
            ),
            React.createElement(
              'div',
              { className: 'deployment-add-credentials__upload twelve-col' },
              'Upload management certificate ›'
            )
          )
        )
      ),
      React.createElement(juju.components.DeploymentPanelFooter, {
        buttons: buttons })
    );
    assert.deepEqual(output, expected);
  });

  it('shows loading message while requesting regions', function () {
    var cloud = clouds['aws'];
    jem.listRegions = function (c, cb) {
      return cb(null, []);
    };
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentAddCredentials, {
      acl: acl,
      changeState: sinon.stub(),
      controller: 'my-controller',
      cloud: cloud,
      jem: jem,
      setDeploymentInfo: sinon.stub(),
      users: users,
      validateForm: sinon.stub().returns(true) }), true);
    var output = renderer.getRenderOutput();
    var props = output.props;
    var emptySelect = React.createElement(
      'select',
      { ref: 'selectRegion',
        disabled: false },
      React.createElement(
        'option',
        null,
        'Loading available regions'
      ),
      null
    );
    assert.deepEqual(props.children[0].props.children[2].props.children[0].props.children[1], emptySelect);
  });

  it('can add the credentials', function () {
    jem.listControllers = function (cb) {
      return cb(null, ['my-controller']);
    };
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentAddCredentials, {
      acl: acl,
      changeState: sinon.stub(),
      controller: 'my-controller',
      cloud: clouds['aws'],
      jem: jem,
      setDeploymentInfo: sinon.stub(),
      users: users,
      validateForm: sinon.stub().returns(true) }), true);
    var instance = renderer.getMountedInstance();
    instance.refs = refs;
    instance._handleAddCredentials();
    assert.equal(jem.addTemplate.callCount, 1);
  });

  it('can select a new cloud', function () {
    var changeState = sinon.stub();
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentAddCredentials, {
      acl: acl,
      changeState: changeState,
      controller: 'my-controller',
      cloud: clouds['aws'],
      jem: jem,
      setDeploymentInfo: sinon.stub(),
      users: users,
      validateForm: sinon.stub().returns(true) }), true);
    var output = renderer.getRenderOutput();
    output.props.children[1].props.buttons[0].action();
    assert.equal(changeState.callCount, 1);
    assert.deepEqual(changeState.args[0][0], {
      sectionC: {
        component: 'deploy',
        metadata: {
          activeComponent: 'choose-cloud'
        }
      }
    });
  });

  it('does not submit the form if there are validation errors', function () {
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentAddCredentials, {
      acl: acl,
      changeState: sinon.stub(),
      controller: 'my-controller',
      cloud: clouds['aws'],
      jem: jem,
      setDeploymentInfo: sinon.stub(),
      users: users,
      validateForm: sinon.stub().returns(false) }), true);
    var instance = renderer.getMountedInstance();
    instance._handleAddCredentials();
    assert.equal(jem.addTemplate.callCount, 0);
  });

  it('disables controls when read only', function () {
    acl.isReadOnly = sinon.stub().returns(true);
    var cloud = clouds['aws'];
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentAddCredentials, {
      acl: acl,
      changeState: sinon.stub(),
      controller: 'my-controller',
      cloud: cloud,
      jem: jem,
      setDeploymentInfo: sinon.stub(),
      users: users,
      validateForm: sinon.stub().returns(true) }), true);
    var instance = renderer.getMountedInstance();
    var output = renderer.getRenderOutput();
    var buttons = [{
      action: instance._handleChangeCloud,
      title: 'Change cloud',
      type: 'inline-neutral'
    }, {
      title: 'Add credential',
      action: instance._handleAddCredentials,
      disabled: true,
      type: 'inline-positive'
    }];
    var expected = React.createElement(
      'div',
      { className: 'deployment-panel__child' },
      React.createElement(
        juju.components.DeploymentPanelContent,
        {
          title: 'Configure Amazon Web Services' },
        React.createElement(
          'div',
          { className: 'deployment-add-credentials__logo' },
          React.createElement(juju.components.SvgIcon, {
            height: cloud.svgHeight,
            name: cloud.id,
            width: cloud.svgWidth })
        ),
        React.createElement(
          'div',
          { className: 'twelve-col deployment-add-credentials__signup' },
          React.createElement(
            'a',
            { href: cloud.signupUrl,
              target: '_blank' },
            'Sign up for ',
            cloud.title,
            ' ',
            React.createElement(juju.components.SvgIcon, {
              name: 'external-link-16',
              size: '12' })
          )
        ),
        React.createElement(
          'form',
          { className: 'twelve-col' },
          React.createElement(
            'div',
            { className: 'six-col' },
            React.createElement(juju.components.DeploymentInput, {
              disabled: true,
              label: 'Credential name',
              placeholder: 'AWS-1',
              required: true,
              ref: 'templateName',
              validate: [{
                regex: /\S+/,
                error: 'This field is required.'
              }, {
                regex: /^[a-zA-Z0-9]([a-zA-Z0-9.-]*[a-zA-Z0-9])?$/,
                error: 'This field must only contain upper and lowercase ' + 'letters, numbers, and hyphens. It must not start or ' + 'end with a hyphen.'
              }] }),
            React.createElement(
              'select',
              { ref: 'selectRegion',
                disabled: true },
              React.createElement(
                'option',
                null,
                'Choose a region'
              ),
              [React.createElement(
                'option',
                { key: 'us-east-1', value: 'us-east-1' },
                'us-east-1'
              )]
            )
          ),
          React.createElement(
            'div',
            { className: 'deployment-panel__notice six-col last-col' },
            React.createElement(
              'p',
              { className: 'deployment-panel__notice-content' },
              React.createElement(juju.components.SvgIcon, {
                name: 'general-action-blue',
                size: '16' }),
              'Credentials are stored securely on our servers and we will notify you by email whenever they are used. See where they are used and manage or remove them via the account page.'
            )
          ),
          React.createElement(
            'h3',
            { className: 'deployment-panel__section-title twelve-col' },
            'Enter credentials'
          ),
          React.createElement(
            'div',
            { className: 'six-col' },
            React.createElement(
              'p',
              { className: 'deployment-add-credentials__p' },
              'You can obtain your AWS credentials at:',
              React.createElement('br', null),
              React.createElement(
                'a',
                { className: 'deployment-panel__link',
                  href: 'https://console.aws.amazon.com/iam/home?region=' + 'eu-west-1#security_credential',
                  target: '_blank' },
                'https://console.aws.amazon.com/iam/home?region=eu-west-1# security_credential'
              )
            ),
            React.createElement(juju.components.DeploymentInput, {
              disabled: true,
              label: 'Access key',
              placeholder: 'TDFIWNDKF7UW6DVGX98X',
              required: true,
              ref: 'templateAccessKey',
              validate: [{
                regex: /\S+/,
                error: 'This field is required.'
              }] }),
            React.createElement(juju.components.DeploymentInput, {
              disabled: true,
              label: 'Secret key',
              placeholder: 'p/hdU8TnOP5D7JNHrFiM8IO8f5GN6GhHj7tueBN9',
              required: true,
              ref: 'templateSecretKey',
              validate: [{
                regex: /\S+/,
                error: 'This field is required.'
              }] })
          )
        )
      ),
      React.createElement(juju.components.DeploymentPanelFooter, {
        buttons: buttons })
    );
    assert.deepEqual(output, expected);
  });
});