/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

var juju = { components: {} }; // eslint-disable-line no-unused-vars

chai.config.includeStack = true;
chai.config.truncateThreshold = 0;

describe('DeploymentChooseCloud', function () {
  var clouds, jem;

  beforeAll(function (done) {
    // By loading this file it adds the component to the juju components.
    YUI().use('deployment-choose-cloud', function () {
      done();
    });
  });

  beforeEach(function () {
    jem = {
      listTemplates: function listTemplates(cb) {
        return cb(null, [{ path: 'test-owner/test' }]);
      },
      listClouds: function listClouds(cb) {
        return cb(null, ['aws']);
      }
    };
    clouds = {
      aws: {
        id: 'aws',
        signupUrl: 'https://portal.aws.amazon.com/gp/aws/developer/' + 'registration/index.html',
        svgHeight: 48,
        svgWidth: 120,
        title: 'Amazon Web Services'
      },
      azure: {
        id: 'azure',
        signupUrl: 'https://azure.microsoft.com/en-us/free/',
        svgHeight: 24,
        svgWidth: 204,
        title: 'Microsoft Azure'
      },
      google: {
        id: 'google',
        signupUrl: 'https://console.cloud.google.com/billing/freetrial',
        svgHeight: 33,
        svgWidth: 256,
        title: 'Google Compute Engine'
      }
    };
  });

  afterEach(function () {
    jem = null;
  });

  it('can render without credentials without clouds', function () {
    jem.listTemplates = function (cb) {
      return cb(null, []);
    };
    jem.listClouds = function (cb) {
      return cb(null, []);
    };
    var pluralize = sinon.stub();
    pluralize.withArgs('application', sinon.match.any).returns('applications');
    pluralize.withArgs('machine', sinon.match.any).returns('machine');
    var get = sinon.stub();
    get.withArgs('name').returns('wordpress');
    get.withArgs('icon').returns('wordpress.svg');
    get.withArgs('id').returns('wordpress1');
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentChooseCloud, {
      changeState: sinon.stub(),
      cloudData: clouds,
      jem: jem,
      setDeploymentInfo: sinon.stub() }), true);
    var output = renderer.getRenderOutput();
    var expected = React.createElement(
      'div',
      { className: 'deployment-panel__child' },
      React.createElement(
        juju.components.DeploymentPanelContent,
        {
          title: 'Choose cloud' },
        React.createElement(
          'div',
          { className: 'deployment-panel__notice twelve-col' },
          React.createElement(
            'p',
            { className: 'deployment-panel__notice-content' },
            React.createElement(juju.components.SvgIcon, {
              name: 'general-action-blue',
              size: '16' }),
            'Add a public cloud credential, and we can save it as an option for later use'
          )
        ),
        React.createElement(
          'div',
          { className: 'deployment-panel__notice twelve-col' },
          React.createElement(
            'p',
            { className: 'deployment-panel__notice-content' },
            React.createElement(juju.components.SvgIcon, {
              name: 'general-action-blue',
              size: '16' }),
            'Fetching available clouds...'
          )
        ),
        React.createElement(
          'div',
          { className: 'deployment-choose-cloud__download twelve-col' },
          React.createElement(juju.components.SvgIcon, {
            height: '30',
            name: 'juju-logo',
            width: '75' }),
          'Deploy manually using Juju to OpenStack, Vmware, MAAS, Joyent or locally to your computer',
          React.createElement(
            'a',
            { className: 'deployment-choose-cloud__download-button ' + 'button--inline-neutral',
              href: 'https://jujucharms.com/docs/stable/reference-releases',
              target: '_blank' },
            'Download Juju'
          )
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can render with credentials without clouds', function () {
    jem.listClouds = function (cb) {
      return cb(null, []);
    };
    var pluralize = sinon.stub();
    pluralize.withArgs('application', sinon.match.any).returns('applications');
    pluralize.withArgs('machine', sinon.match.any).returns('machine');
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentChooseCloud, {
      changeState: sinon.stub(),
      cloudData: clouds,
      jem: jem,
      setDeploymentInfo: sinon.stub() }), true);
    var output = renderer.getRenderOutput();
    var expected = React.createElement(
      'div',
      { className: 'deployment-panel__child' },
      React.createElement(
        juju.components.DeploymentPanelContent,
        {
          title: 'Choose cloud or saved credential' },
        React.createElement(
          'div',
          null,
          React.createElement(
            'h3',
            { className: 'deployment-panel__section-title twelve-col' },
            'Saved cloud credentials'
          ),
          React.createElement(
            'ul',
            { className: 'deployment-choose-cloud__list twelve-col' },
            [React.createElement(
              'li',
              { className: 'deployment-choose-cloud__cloud-option ' + 'deployment-choose-cloud__cloud-option--credential six-col',
                key: 'test-owner/test',
                onClick: output.props.children.props.children[0].props.children[1].props.children[0].props.onClick },
              React.createElement(
                'span',
                { className: 'deployment-choose-cloud__cloud-option-title' },
                React.createElement(
                  'span',
                  { className: 'deployment-choose-cloud__cloud-option-name' },
                  'test'
                ),
                React.createElement(
                  'span',
                  { className: 'deployment-choose-cloud__cloud-option-owner' },
                  'test-owner'
                )
              )
            )]
          ),
          React.createElement(
            'h3',
            { className: 'deployment-panel__section-title twelve-col' },
            'Public clouds'
          )
        ),
        React.createElement(
          'div',
          { className: 'deployment-panel__notice twelve-col' },
          React.createElement(
            'p',
            { className: 'deployment-panel__notice-content' },
            React.createElement(juju.components.SvgIcon, {
              name: 'general-action-blue',
              size: '16' }),
            'Fetching available clouds...'
          )
        ),
        React.createElement(
          'div',
          { className: 'deployment-choose-cloud__download twelve-col' },
          React.createElement(juju.components.SvgIcon, {
            height: '30',
            name: 'juju-logo',
            width: '75' }),
          'Deploy manually using Juju to OpenStack, Vmware, MAAS, Joyent or locally to your computer',
          React.createElement(
            'a',
            { className: 'deployment-choose-cloud__download-button ' + 'button--inline-neutral',
              href: 'https://jujucharms.com/docs/stable/reference-releases',
              target: '_blank' },
            'Download Juju'
          )
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can render without credentials and with clouds', function () {
    jem.listTemplates = function (cb) {
      return cb(null, []);
    };
    var pluralize = sinon.stub();
    pluralize.withArgs('application', sinon.match.any).returns('applications');
    pluralize.withArgs('machine', sinon.match.any).returns('machine');
    var get = sinon.stub();
    get.withArgs('name').returns('wordpress');
    get.withArgs('icon').returns('wordpress.svg');
    get.withArgs('id').returns('wordpress1');
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentChooseCloud, {
      changeState: sinon.stub(),
      cloudData: clouds,
      jem: jem,
      setDeploymentInfo: sinon.stub() }), true);
    var output = renderer.getRenderOutput();
    var expected = React.createElement(
      'div',
      { className: 'deployment-panel__child' },
      React.createElement(
        juju.components.DeploymentPanelContent,
        {
          title: 'Choose cloud' },
        React.createElement(
          'div',
          { className: 'deployment-panel__notice twelve-col' },
          React.createElement(
            'p',
            { className: 'deployment-panel__notice-content' },
            React.createElement(juju.components.SvgIcon, {
              name: 'general-action-blue',
              size: '16' }),
            'Add a public cloud credential, and we can save it as an option for later use'
          )
        ),
        React.createElement(
          'ul',
          { className: 'deployment-choose-cloud__list twelve-col' },
          [React.createElement(
            'li',
            { className: 'deployment-choose-cloud__cloud-option four-col ',
              key: 'aws',
              onClick: output.props.children.props.children[1].props.children[0].props.onClick },
            React.createElement(
              'span',
              { className: 'deployment-choose-cloud__cloud-option-image' },
              React.createElement(juju.components.SvgIcon, {
                height: clouds['aws'].svgHeight,
                name: clouds['aws'].id,
                width: clouds['aws'].svgWidth })
            )
          ), React.createElement(
            'li',
            { className: 'deployment-choose-cloud__cloud-option four-col ',
              key: 'azure',
              onClick: output.props.children.props.children[1].props.children[1].props.onClick },
            React.createElement(
              'span',
              { className: 'deployment-choose-cloud__cloud-option-image' },
              React.createElement(juju.components.SvgIcon, {
                height: clouds['azure'].svgHeight,
                name: clouds['azure'].id,
                width: clouds['azure'].svgWidth })
            )
          ), React.createElement(
            'li',
            { className: 'deployment-choose-cloud__cloud-option four-col last-col',
              key: 'google',
              onClick: output.props.children.props.children[1].props.children[2].props.onClick },
            React.createElement(
              'span',
              { className: 'deployment-choose-cloud__cloud-option-image' },
              React.createElement(juju.components.SvgIcon, {
                height: clouds['google'].svgHeight,
                name: clouds['google'].id,
                width: clouds['google'].svgWidth })
            )
          )]
        ),
        React.createElement(
          'div',
          { className: 'deployment-choose-cloud__download twelve-col' },
          React.createElement(juju.components.SvgIcon, {
            height: '30',
            name: 'juju-logo',
            width: '75' }),
          'Deploy manually using Juju to OpenStack, Vmware, MAAS, Joyent or locally to your computer',
          React.createElement(
            'a',
            { className: 'deployment-choose-cloud__download-button ' + 'button--inline-neutral',
              href: 'https://jujucharms.com/docs/stable/reference-releases',
              target: '_blank' },
            'Download Juju'
          )
        )
      )
    );
    assert.deepEqual(output, expected);
  });

  it('can navigate from a credential', function () {
    var changeState = sinon.stub();
    var setDeploymentInfo = sinon.stub();
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentChooseCloud, {
      changeState: changeState,
      cloudData: clouds,
      jem: jem,
      setDeploymentInfo: setDeploymentInfo }), true);
    var output = renderer.getRenderOutput();
    output.props.children.props.children[0].props.children[1].props.children[0].props.onClick();
    assert.equal(changeState.callCount, 1);
    // It should store the template name in the parentId
    assert.equal(setDeploymentInfo.callCount, 1);
    assert.deepEqual(setDeploymentInfo.args[0], ['templateName', 'test-owner/test']);
    assert.deepEqual(changeState.args[0][0], {
      sectionC: {
        component: 'deploy',
        metadata: {
          activeComponent: 'summary'
        }
      }
    });
  });

  it('can select a new cloud', function () {
    var changeState = sinon.stub();
    var setDeploymentInfo = sinon.stub();
    var renderer = jsTestUtils.shallowRender(React.createElement(juju.components.DeploymentChooseCloud, {
      changeState: changeState,
      cloudData: clouds,
      jem: jem,
      setDeploymentInfo: setDeploymentInfo }), true);
    var output = renderer.getRenderOutput();
    output.props.children.props.children[1].props.children[0].props.onClick();
    assert.equal(changeState.callCount, 1);
    assert.deepEqual(setDeploymentInfo.args[0], ['cloud', 'aws']);
    assert.deepEqual(changeState.args[0][0], {
      sectionC: {
        component: 'deploy',
        metadata: {
          activeComponent: 'add-credentials-aws'
        }
      }
    });
  });
});