/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.
This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.
This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.
You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('inspector-relations', function () {

  juju.components.InspectorRelations = React.createClass({

    propTypes: {
      acl: React.PropTypes.object.isRequired,
      changeState: React.PropTypes.func.isRequired,
      destroyRelations: React.PropTypes.func.isRequired,
      serviceRelations: React.PropTypes.array.isRequired
    },

    /**
      Generate the initial state.
       @method getInitialState
      @returns {String} The intial state.
    */
    getInitialState: function getInitialState() {
      return { activeCount: 0 };
    },

    /**
      Update the count of the number of active checkboxes.
       @method _updateActiveCount
    */
    _updateActiveCount: function _updateActiveCount() {
      var activeCount = 0;
      var refs = this.refs;
      Object.keys(refs).forEach(function (ref) {
        if (ref.split('-')[0] === 'CheckListItem') {
          if (refs[ref].state.checked) {
            activeCount += 1;
          }
        }
      });
      this.setState({ 'activeCount': activeCount });
    },

    /**
      Generate the relation label.
       @method _generateRelationLabel
      @param {Object} The relation object.
      @returns {String} The relation label.
    */
    _generateRelationLabel: function _generateRelationLabel(relation) {
      var serviceName = relation.far.serviceName;
      var relationName = relation.far.name;
      return serviceName + ':' + relationName;
    },

    /**
      Generate the relation list of components.
       @method _generateRelations
      @returns {Object} The relation components.
    */
    _generateRelations: function _generateRelations() {
      var relations = this.props.serviceRelations;
      var disabled = this.props.acl.isReadOnly();
      var activeRelations = [];
      // Remove deleted relations from the list
      for (var i in relations) {
        if (relations.hasOwnProperty(i) && !relations[i].deleted) {
          activeRelations.push(relations[i]);
        }
      }
      relations = activeRelations;
      if (relations.length === 0) {
        return React.createElement(
          'li',
          { className: 'inspector-relations__message' },
          'No active relations for this application.'
        );
      }
      var ref = 'select-all';
      var components = [React.createElement(juju.components.CheckListItem, {
        className: 'select-all',
        disabled: disabled,
        key: ref + '1',
        ref: ref,
        label: 'Select all relations',
        whenChanged: this._selectAllRelations })];

      relations.forEach(function (relation, index) {
        var ref = 'CheckListItem-' + relation.id;
        components.push(React.createElement(juju.components.CheckListItem, {
          action: this._handleRelationClick.bind(this, index),
          disabled: disabled,
          key: relation.id,
          ref: ref,
          label: this._generateRelationLabel(relation),
          relation: relation,
          changeState: this.props.changeState,
          whenChanged: this._updateActiveCount }));
      }, this);
      return components;
    },

    /**
      Handle navigating to a relation details when it is clicked.
       @method _handleRelationClick
    */
    _handleRelationClick: function _handleRelationClick(index) {
      // Cast to string to pass state null check
      index = index + '';
      this.props.changeState({
        sectionA: {
          component: 'inspector',
          metadata: {
            activeComponent: 'relation',
            unit: index
          }
        }
      });
    },

    /**
      Remove the selected relations
       @method _handleRemoveRelation
    */
    _handleRemoveRelation: function _handleRemoveRelation() {
      var relations = [];
      var refs = this.refs;
      Object.keys(refs).forEach(function (ref) {
        var isInstance = ref.split('-')[0] === 'CheckListItem';
        if (isInstance && refs[ref].state.checked) {
          var relationName = ref.slice(ref.indexOf('-') + 1);
          relations.push(relationName);
        }
      });
      this.props.destroyRelations(relations);
      this._selectAllRelations(false);
    },

    /**
      Sets the selectAll state property based on the "select all" child
      component.
       @method _selectAllRelations
      @param {Boolean} checked Whether to check or uncheck the relations.
    */
    _selectAllRelations: function _selectAllRelations(checked) {
      var _this = this;

      var refs = this.refs;
      Object.keys(refs).forEach(function (ref) {
        if (ref.split('-')[0] === 'CheckListItem') {
          refs[ref].setState({
            checked: checked
          }, function () {
            // After the state has been updated then update the active unit
            // count to enable/disable the buttons.
            _this._updateActiveCount();
          });
        }
      });
    },

    /**
      Generate the remove button.
       @method _generateButtons
      @returns {Object} The button row component.
    */
    _generateButtons: function _generateButtons() {
      if (this.props.serviceRelations.length === 0) {
        return;
      }
      var disabled = this.state.activeCount === 0 || this.props.acl.isReadOnly();
      var buttons = [];
      buttons.push({
        title: 'Remove',
        type: 'neutral',
        action: this._handleRemoveRelation,
        disabled: disabled
      });
      return React.createElement(juju.components.ButtonRow, {
        buttons: buttons });
    },

    render: function render() {
      return React.createElement(
        'div',
        { className: 'inspector-relations' },
        React.createElement(
          'ul',
          { className: 'inspector-relations__list' },
          this._generateRelations()
        ),
        this._generateButtons()
      );
    }

  });
}, '0.1.0', { requires: ['check-list-item', 'button-row'] });